@@ -8,7 +8,7 @@
 #
 ###############################################################################
 #
-#   $Id: Procedure.pm,v 1.4 2002/05/03 21:07:38 rjray Exp $
+#   $Id: Procedure.pm,v 1.5 2002/05/22 09:45:59 rjray Exp $
 #
 #   Description:    This class abstracts out all the procedure-related
 #                   operations from the RPC::XML::Server class
@@ -50,7 +50,7 @@ use subs qw(new is_valid name code signature help version hidden
 use AutoLoader 'AUTOLOAD';
 require File::Spec;
 
-$VERSION = do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
 
 1;
 
@@ -880,7 +880,9 @@ sub load_XPL_file
 #
 #   Sub Name:       call
 #
-#   Description:    
+#   Description:    Encapsulates the invocation of the code block that the
+#                   object is abstracting. Manages parameters, signature
+#                   checking, etc.
 #
 #   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
 #                   $self     in      ref       Object of this class
@@ -911,11 +913,10 @@ sub call
     $resptype = $self->match_signature($signature);
     # Since there must be at least one signature with a return value (even
     # if the param list is empty), this tells us if the signature matches:
-    return RPC::XML::response
-        ->new(RPC::XML::fault->new(301,
-                                   "method $name nas no matching " .
-                                   'signature for the argument list'))
-            unless ($resptype);
+    return RPC::XML::fault->new(301,
+                                "method $name nas no matching " .
+                                'signature for the argument list')
+        unless ($resptype);
 
     # Set these in case the server object is part of the param list
     local $srv->{signature} = [ $resptype, @paramtypes ];
